#!/usr/bin/env python3

"""
A modified version of `card.py` that simulates a bank card up until a nonce is
received from the terminal. The nonce is printed to the screen, and the process
is repeated.
"""

import arguments

import binascii
import re
import socket
import sys


def receive(s):
    received_length = s.recv(4)
    length = int.from_bytes(received_length, byteorder='big', signed=True)
    raw_response = s.recv(length)
    response = binascii.hexlify(bytearray(raw_response))
    fmt_response = response.decode('utf-8').upper()
    return fmt_response


def send(command, s):
    command = command.upper()
    cmd_bytes = bytearray.fromhex(command)
    length = len(cmd_bytes)
    s.sendall(length.to_bytes(4, byteorder='big', signed=True))
    s.sendall(cmd_bytes)


if __name__ == "__main__":
    # Socket for terminal and card.
    settings = arguments.parse(1)
    term_conn = settings.conns[0]

    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as term:
        term.connect((term_conn['host'], term_conn['port']))

        while True:
            cmd = receive(term)
            if cmd == '':
                print("Empty")
                break

            # Match the 'Get Processing Options' command.
            c = re.match('80A8000023832136A0400000000000010000000000000008260000000000082619....00(........)00', cmd)
            # If the command receieved was an 'Application Query'...
            if re.match("00A404000E325041592E5359532E444446303100", cmd):
                # ...send a list of applications our fake card supports.
                send("6F3A840E325041592E5359532E4444463031A528BF0C2561234F07A0000000031010500A566973612044656269748701039F0A0800010501000000009000", term)
            # If the command receieved was an 'Application Select'...
            elif re.match("00A4040007A000000003101000", cmd):
                # ...send a list processing details required from the terminal
                # for a transaction of this type.
                send("6F528407A0000000031010A547500A566973612044656269748701039F38189F66049F02069F03069F1A0295055F2A029A039C019F3704BF0C1A9F5A0531082608269F0A080001050100000000BF6304DF2001809000", term)
            # If the command received was 'Get Processing Options'
            elif c:
                # Print out the nonce we received.
                print(c.group(1))
                # Send back the response to the original 'Application Query',
                # thereby resetting the transaction back to the start.
                send("6F3A840E325041592E5359532E4444463031A528BF0C2561234F07A0000000031010500A566973612044656269748701039F0A0800010501000000009000", term)
            else:
                # We don't expect any commands except the above.
                print("ERROR!! cmd was " + cmd)
                break


        print("Finished.")
